import argparse

from lofter_utils.TagConverterHelper import convert_file, convert_dir
from lofter_utils.TagDownloadHelper import download_line, download_file, download_dir


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--action', required=True, choices=['download', 'convert'])
    parser.add_argument('--data-type', required=True, choices=['line', 'file', 'dir'])
    parser.add_argument('--data-value', required=True)
    parser.add_argument('--meta-type', choices=['tag'], default='tag')
    parser.add_argument('--add-dir-timestamp', action='store_true')
    parser.add_argument('--add-file-timestamp', action='store_true')
    parser.add_argument('--output-dir')
    args = parser.parse_args()
    if args.action == 'download':
        if args.data_type == 'line':
            download_line(args.data_value, output_dir=args.output_dir)
        elif args.data_type == 'file':
            download_file(args.data_value, add_dir_timestamp=args.add_dir_timestamp,
                          output_dir=args.output_dir,
                          add_file_timestamp=args.add_file_timestamp)
        elif args.data_type == 'dir':
            download_dir(args.data_value, add_dir_timestamp=args.add_dir_timestamp,
                         output_dir=args.output_dir,
                         add_file_timestamp=args.add_file_timestamp)
    elif args.action == 'convert':
        if args.data_type == 'file':
            convert_file(args.data_value, output_dir=args.output_dir)
        elif args.data_type == 'dir':
            convert_dir(args.data_value, output_dir=args.output_dir)
        else:
            print('Illegal param: --data-type=%s' % args.data_type)
            exit(1)
    else:
        print('Illegal param: --action=%s' % args.action)
        exit(1)


if __name__ == '__main__':
    main()



# -*- coding: UTF-8 –*-

import os
import time

from lofter_utils.TagDownloader import downloadTag


def download_line(line, add_file_timestamp=False, output_dir='.'):
    line = line.replace('\r', '')
    line = line.replace('\n', '')
    allTypes = ['new', 'total', 'mouth', 'week', 'date']
    print('开始下载line: "%s".' % line)
    params = line.split('\t')
    for i in range(4 - len(params)):
        params.append(None)
    tag, types, limit, offset = params
    if tag == None:
        print('跳过空行')
        return
    if types == None:
        types = allTypes
    else:
        types = types.split(',')
    if limit == None:
        limit = 500
    else:
        limit = int(limit)
    if offset == None:
        offset = 0
    else:
        offset = int(offset)
    if not os.path.exists(output_dir):
        os.mkdir(output_dir)
    for type in types:
        try:
            pages = downloadTag(tag, type, limit, offset)
        except Exception as e:
            print('下载pages出错: tag = %s, type = %d, limit = %d, offset = %d' % (tag, type, limit, offset))
            print(e)
            continue
        for page in pages:
            filename = 'tag_%s_type_%s_limit_%s_offset_%s.tag' % (
                page['tag'], page['type'], page['limit'], page['offset'])
            if add_file_timestamp:
                tmp_1 = os.path.splitext(filename)
                timestamp = time.strftime('%Y%m%d%H%M%S', time.localtime())
                filename = tmp_1[0] + '_' + timestamp + tmp_1[1]
            filename = os.path.join(output_dir, filename)
            with open(filename, 'wb') as f:
                f.write(page['body'])


def download_file(filename, add_dir_timestamp=False, add_file_timestamp=False, output_dir=None):
    print('开始下载taglist文件: "%s".' % filename)
    tmp_1 = os.path.splitext(filename)
    if tmp_1[1] == '.' + 'taglist':
        out_dir = tmp_1[0]
    else:
        out_dir = filename
    if output_dir != None:
        out_dir = output_dir
    if add_dir_timestamp:
        timestamp = time.strftime('%Y%m%d%H%M%S', time.localtime())
        out_dir = out_dir + '_' + timestamp
    with open(filename, encoding='utf8') as f:
        for line in f:
            try:
                download_line(line, add_file_timestamp=add_file_timestamp, output_dir=out_dir)
            except Exception as e:
                print('下载line时出错')
                print(e)
                continue
    print('taglist文件中tag已下载至: "%s"' % out_dir)


def download_dir(dirname, tool, add_dir_timestamp=False, add_file_timestamp=False, output_dir=None):
    print('开始下载taglist目录: "%s".' % dirname)
    for filename in os.listdir(dirname):
        filename = os.path.join(dirname, filename)
        if not os.path.isfile(filename):
            continue
        if os.path.splitext(filename)[1] != '.' + 'taglist':
            continue
        download_file(filename, add_dir_timestamp=add_dir_timestamp, add_file_timestamp=add_file_timestamp,
                      output_dir=output_dir)
    print('taglist目录中taglist文件已下载至: "%s"' % dirname)



# -*- coding: UTF-8 –*-
import os
import re

from lofter_utils.TagConverter import processPage


def processTagFileList(filenames):
    res = {}
    for filename in filenames:
        filename = os.path.split(filename)[1]
        subFilename = re.match(r"^tag_(.+)_type_(.+)_limit_(\d+)_offset_(\d+)(_.*)?\.tag", filename)
        tag = subFilename.group(1)
        type = subFilename.group(2)
        limit = int(subFilename.group(3))
        offset = int(subFilename.group(4))
        suffix = subFilename.group(5)
        key = (tag, type, limit, suffix)
        if not key in res:
            value = []
            res[key] = value
        else:
            value = res[key]
        value.append(offset)
        res[key] = value
    for key in res:
        res[key].sort()
    return res


def isTagFile(filename):
    if not os.path.isfile(filename):
        return False
    if os.path.splitext(filename)[1] != '.tag':
        return False
    return True


def writeArticlesToFile(articles, f):
    for article in articles:
        blogPageUrl = article['blogPageUrl']
        blogPageUrl = '# ' + blogPageUrl + '\r\n'
        f.write(blogPageUrl.encode('utf-8'))
        content = article['content']
        try:
            content += '\r\n'
            content = content.encode('utf-8').decode("raw_unicode_escape").encode('utf-16', 'surrogatepass').decode('utf-16')
            f.write(content.encode('utf-8'))
        except Exception as e:
            print('content转码出现问题')
            print(e)
            f.write('content转码出现问题\r\n'.encode('utf-8'))
            continue


def convert_file(filename, output_dir=None):
    print('开始转换文件: "%s".' % filename)
    # Determine output file name
    tmp_1 = os.path.splitext(filename)
    if tmp_1[1] == '.' + 'taglist':
        out_filename = tmp_1[0] + '.' + 'txt'
    else:
        out_filename = filename + '.' + 'txt'
    if output_dir != None:
        if not os.path.exists(output_dir):
            os.mkdir(output_dir)
        out_filename = os.path.join(output_dir, os.path.split(out_filename)[1])
    with open(out_filename, 'w', encoding='utf-8', errors='surrogatepass') as out_f:
        with open(filename, 'r', encoding='utf-8') as f:
            str = f.read()
            articles = processPage(str)
            writeArticlesToFile(articles, out_f)


def convert_dir(dir, output_dir=None):
    filenames = os.listdir(dir)
    filenames = map(lambda x: os.path.join(dir, x), filenames)
    filenames = filter(isTagFile, filenames)
    fileTypeList = processTagFileList(filenames)
    for key in fileTypeList:
        print(key)
        tag, type, limit, suffix = key
        if suffix == None:
            suffix = ''
        out_filename = 'tag_%s_type_%s_limit_%d%s.txt' % (tag, type, limit, suffix)
        out_filename = os.path.join(dir, out_filename)
        print('outFilename = %s' % out_filename)
        with open(out_filename, 'wb') as out_f:
            for offset in fileTypeList[key]:
                in_filename = 'tag_%s_type_%s_limit_%d_offset_%d%s.tag' % (tag, type, limit, offset, suffix)
                in_filename = os.path.join(dir, in_filename)
                print('in_filename = %s' % in_filename)
                with open(in_filename, 'r', encoding='utf-8') as in_f:
                    str = in_f.read()
                    articles = processPage(str)
                    writeArticlesToFile(articles, out_f)



# -*- coding: UTF-8 –*-

import re
import os


def processPage(page):
    print('processPage()')
    blocks = page.split('\n\n')
    print('len(blocks) = %d' % len(blocks))
    articles = []
    i = 0
    for block in blocks:
        article = {}
        blogPageUrls = re.findall('\\.blogPageUrl="(.+?)(?<!\\\\)"', block)
        if len(blogPageUrls) == 0:
            hasBlogPageUrls = False
            print('block[i], i = %d' % i)
            blogPageUrl = 'blogPageUrl未找到'
            print(blogPageUrl)
            print(block)
        else:
            hasBlogPageUrls = True
            blogPageUrl = blogPageUrls[0]
        contents = re.findall('\\.content="(.+?)(?<!\\\\)"', block)
        if len(contents) == 0:
            hasContents = False
            print('block[i], i = %d' % i)
            content = 'content未找到'
            print(content)
            print(block)
        else:
            hasContents = True
            content = contents[0]
        if (not hasBlogPageUrls) and (not hasContents):
            continue
        article['blogPageUrl'] = blogPageUrl
        article['content'] = content
        articles.append(article)
        i += 1
    return articles


def processTagFileList(filenames):
    res = {}
    for filename in filenames:
        subFilename = re.match(r"^tag_(.+)_type_(.+)_limit_(\d+)_offset_(\d+)\.tag", filename)
        tag = subFilename.group(1)
        type = subFilename.group(2)
        limit = int(subFilename.group(3))
        offset = int(subFilename.group(4))
        key = (tag, type, limit)
        if not key in res:
            value = []
            res[key] = value
        else:
            value = res[key]
        value.append(offset)
        res[key] = value
    for key in res:
        res[key].sort()
    return res


def isTagFile(filename):
    if not os.path.isfile(filename):
        return False
    if os.path.splitext(filename)[1] != '.tag':
        return False
    return True



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="lofter_utils",
    version="0.1.1",
    author="Hawthorn2013",
    author_email="hawthorn7dd@hotmail.com",
    description="Lofter utils support download and convert danmu.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Hawthorn2013/lofter_utils",
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': ['lofterutils=lofter_utils.CmdInterface:main'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


# -*- coding: UTF-8 –*-

import requests
from urllib.parse import quote
import re


def hasContent(str):
    blogPageUrls = re.findall('\\.blogPageUrl="(.+?)(?<!\\\\)"', str)
    return len(blogPageUrls) > 0


def downloadPage(tag, type, limit, offset):
    print("downloadPage(), tag = %s, type = %s, limit = %d, offset = %d" % (tag, type, limit, offset))
    tag = quote(tag, 'utf-8')
    type = quote(type, 'utf-8')
    url = 'http://www.lofter.com/dwr/call/plaincall/TagBean.search.dwr'
    referer = 'http://www.lofter.com/tag/%s/%s' % (tag, type)
    body = '''callCount=1
scriptSessionId=${scriptSessionId}187
httpSessionId=
c0-scriptName=TagBean
c0-methodName=search
c0-id=0
c0-param0=string:%s
c0-param1=number:0
c0-param2=string:
c0-param3=string:%s
c0-param4=boolean:false
c0-param5=number:0
c0-param6=number:%s
c0-param7=number:%s
c0-param8=number:0
batchId=978839''' % (tag, type, str(limit), str(offset))
    res = requests.post(url, data=body, headers={'Referer': referer})
    return res.content


def downloadTag(tag, type, limit=500, offset=0):
    print("downloadTag(), tag = %s, type = %s" % (tag, type))
    res = []
    while True:
        page = {}
        body = downloadPage(tag, type, limit, offset)
        if not hasContent(body.decode('utf-8')):
            break
        page['tag'] = tag
        page['type'] = type
        page['limit'] = limit
        page['offset'] = offset
        page['body'] = body
        res.append(page)
        offset += limit
    return res


def downloadTagWithTypes(tag, types=None):
    defaultTypes = ['new', 'total', 'mouth', 'week', 'date']
    if types == None:
        types = defaultTypes
    res = []
    for type in types:
        pages = downloadTag(tag, type)
        res.extend(pages)
    return res


if __name__ == '__main__':
    tags = []
    for tag in tags:
        pages = downloadTagWithTypes(tag)
        for page in pages:
            filename = 'tag_%s_type_%s_limit_%s_offset_%s.tag' % (
                page['tag'], page['type'], page['limit'], page['offset'])
            with open(filename, 'wb') as f:
                f.write(page['body'])
